;Stub standard-I/O filter, for the Win32 console
;Also illustrates buffered file input and output

comment~ Sample build file:
@echo off
\masm32\bin\ml /c /coff stdio.asm
if errorlevel 1 goto end
\masm32\bin\polink /SUBSYSTEM:CONSOLE /STACK:4096,4096 /HEAP:4096,4096 stdio.obj
del stdio.obj
rem dosstub stdio.exe    optional
dir stdio.exe
:end
~

comment~ When it's assembled, enter e.g.
stdio<sometextfile
or
echo some text|stdio
or
type sometextfile|stdio
and you will see the input capitalized. The output can be saved
to a file:
stdio<sometextfile>newfilespec
echo some text|stdio>newfilespec

Filters of this kind can be chained together, as in:
type sometextfile|caps.exe|ltrim.exe
where ltrim is a filter which removes leading spaces from a text line.
~

.586
.model flat, stdcall
option casemap:none

include \masm32\include\kernel32.inc
includelib \masm32\lib\kernel32.lib

ReadBuff_ equ 200h     ;buffer sizes
WriteBuff_ equ 200h

.data
ExitAddress  dd ?
hRead        dd ?    ;file handle
ReadBuff     dd ?    ;address of a buffer in the stack
amtReadBuff  dd 0    ;number of bytes of data remaining in the buffer
ptrReadBuff  dd ?    ;address of the next byte to get
hWrite       dd ?    ;file handle
WriteBuff    dd ?    ;address of a buffer in the stack
amtWriteBuff dd WriteBuff_    ;number of bytes of space remaining in the buffer
ptrWriteBuff dd ?    ;address to which to put the next byte

.code
Start:
    pop ExitAddress    ;an alternative to ExitProcess or ExitThread
    sub esp,ReadBuff_  ;heap instead of stack could be used for these buffers
    mov ReadBuff,esp
    sub esp,WriteBuff_
    mov WriteBuff,esp
    mov ptrWriteBuff,esp
    invoke GetStdHandle,-10   ; -10 = STD_INPUT_HANDLE
    mov hRead,eax
    invoke GetStdHandle,-11   ; -11 = STD_OUTPUT_HANDLE
    mov hWrite,eax

MainLoop:
    call GetByte
    js InputExhausted
       ; now we do something with the input, say capitalize it
    cmp al,"a"
    jb short @F
    cmp al,"z"+1
    jnb short @F
    and al,0DFh   ;if you want to spell it out: and al,NOT("A" XOR "a")  :)
@@:
    call PutByte
    jmp MainLoop

InputExhausted:
    call FlushWrite
    xor eax,eax
    jmp ExitAddress

GetByte:  ;returns a byte in AL, from the read handle, or SF if end-of-file.
    mov esi,offset amtReadBuff
    dec dword ptr[esi]
    jns short @F   ;if read buffer is empty, read more of the input
    invoke ReadFile,hRead,ReadBuff,ReadBuff_,esi,0
    mov eax,[esi]
    dec eax
    js short AnyRet
    mov [esi],eax
    mov eax,ReadBuff
    mov ptrReadBuff,eax
@@: mov esi,ptrReadBuff
    lodsb
    mov ptrReadBuff,esi
AnyRet: ret

PutByte:      ;input is al
    mov edi,ptrWriteBuff
    stosb
    mov ptrWriteBuff,edi
    dec amtWriteBuff
    jnz short AnyRet   ;if write buffer is now full, write what is there
FlushWrite:
    mov eax,ptrWriteBuff
    mov edx,WriteBuff
    sub eax,edx
    invoke WriteFile,hWrite,edx,eax,addr amtWriteBuff,0
    mov amtWriteBuff,WriteBuff_
    mov eax,WriteBuff
    mov ptrWriteBuff,eax
    ret

end Start
